#ifndef __ALCHEMIST__
#define __ALCHEMIST__

#include <sys/types.h> /* for: uid_t, gid_t */

/*
 * alchemist.h
 */

/* bump the major version every time a change made which would break existing
* compiled wodules. */
#define ALCHEMIST_ABI_MAJOR_VER 0

/* bump the minor version every time a change is made which would not break
* existing modules with an equivalent major and equall or smaller minor version
*/
#define ALCHEMIST_ABI_MINOR_VER 0

typedef struct AdmError AdmError;

/**
 * Constructor/allocator for our AdmError structure. Fails safely and silently if error is NULL.
 * @brief        Allocate new AdmError structure
 * @param error  The pointer to the AdmError pointer in which the newly
 *               allocated AdmError structure will be returned.
 * @param num    The integer error code of the new AdmError structure.
 * @param format The output format in printf(3) format.
 * @param ...	 The variable arguments for the printf(3) format.
 */
void		AdmError_create		(AdmError	**error,
					 int 		  num,
					 const char	 *format,
					 		  ...);

/**
 * @brief       Increment the ref count of an AdmError structure
 * @param error The pointer to the AdmError structure to be deallocated.
 */
void		 AdmError_ref		(AdmError        *error);

/**
 * Decrements the reference count of the given AdmError function, and deallocates
 * it when the count reaches 0. This function must only be called by reference holders.
 * @brief       Decrement the ref count of an AdmError structure
 * @param error The pointer to the AdmError structure to be deallocated.
 */
void		 AdmError_unref		(AdmError        *error);

/**
 * Returns the error string of the given AdmError structure.
 * The error string returned is guaranteed for only as long as the AdmError is refed.
 * Users that want a copy, should make one.
 * @brief        String description of error
 * @param  error The pointer to the AdmError structure.
 * @return       The character string that contains the description of the
 *               error of the given AdmError structure.
 * @note         Based on the strerror(3) *NIX C library.
 */
const char	*AdmError_strerr	(AdmError        *error);

/**
 * Returns the error code of the given AdmError structure.
 * @brief        Integer error code of error
 * @param  error The pointer to the AdmError structure.
 * @return       The @c int error code of the given AdmError structure. See
 *               admerror.h for a list of valid error code.
 * @note         Based on errno(3) *NIX C library.
 * @see          AdmError
 */
int		 AdmError_errno		(AdmError        *error);

/* AdmError Numbers */
#define ADM_EDEADNODE		1	/** The node has been unlinked **/
#define ADM_EBADTYPE		2	/** The node was not the correct type for this
					    opperation **/
#define ADM_EBADNAME		3	/** The name used is not valid (illegal chars) **/
#define ADM_EBADDATA		4	/** Bad data was passed as input **/
#define ADM_ECONSTRAINT		5	/** A data constraint was violated **/

#define ADM_ENOINDEX		51	/** The reqeusted index is out of range **/
#define ADM_ENONAME		52	/** The reqeusted name was not found **/
#define ADM_ENOPATH		53	/** The reqeusted path was not found **/

#define ADM_EACCESSFAILED	101	/** The access failed **/
#define ADM_ENONAMESPACE	102	/** The namespace could not be found **/
#define ADM_ENOSUBSPACE		103	/** The subspace could not be found **/

#define ADM_ENOWRITE		201	/** This AdmBlackBox does not support writes **/
#define ADM_ENOREAD		202	/** This AdmBlackBox does not support reads **/
#define ADM_ENOLISTSERIAL	203	/** This AdmBlackBox does not support listSerial **/
#define ADM_ENOREADSERIAL	204	/** This AdmBlackBox does not support readSerial **/
#define ADM_ENOCACHE		205	/** This AdmBlackBox is not being cached **/

#define ADM_ENOBOXTYPE		251	/** The boxtype in question could not be found **/

#define ADM_EPARSE		301	/** The xml parse failed **/

typedef struct AdmRefStr AdmRefStr;

AdmRefStr *	AdmRefStr_ref		(AdmRefStr	*refstr);
void		AdmRefStr_unref		(AdmRefStr	*refstr);

size_t		AdmRefStr_len		(AdmRefStr	*refstr);

#define AdmRefStr_str(REFSTR) \
({ \
 	AdmRefStr *__tok = (REFSTR); \
	(const char *) __tok; \
})

/**
* @brief	Get an AdmRefStr
* @param	str	the string to match the AdmRefStr to
* @param	force	if we should create a matching AdmRefStr if one does
*			not already exist
* @return	the unique matching AdmRefStr, or @c NULL if there is no match
*/
AdmRefStr *	AdmRefStr_get		(const char	*str,
					 int		 force);

/**
* @brief	Get an AdmRefStr, by length
* @param	str	the string to match the AdmRefStr to
* @param	len	the lenght of the string, as strlen would report it
* @param	force	if we should create a matching AdmRefStr if one does
*			not already exist
* @return	the unique matching AdmRefStr, or @c NULL if there is no match
*/
AdmRefStr *	AdmRefStr_nget		(const char	*str,
					 size_t		 len,
					 int		 force);

typedef struct AdmData AdmData;

typedef struct AdmList AdmList;

typedef struct AdmInt AdmInt;

typedef struct AdmBool AdmBool;

typedef struct AdmFloat AdmFloat;

typedef struct AdmString AdmString;

typedef struct AdmString AdmBase64;

typedef struct AdmString AdmCopy;

typedef struct AdmIdentity AdmIdentity;

typedef struct AdmContext AdmContext;

typedef enum {
        ADM_TYPE_UNKNOWN = 0,
        ADM_TYPE_LIST    = 1,
        ADM_TYPE_COPY    = 2,
        ADM_TYPE_INT     = 3,
        ADM_TYPE_FLOAT   = 4,
        ADM_TYPE_BOOL    = 5,
        ADM_TYPE_STRING  = 6,
        ADM_TYPE_BASE64  = 7
} AdmDataType;

/* context create and destroy */
/**
 * AdmContext_new: create a new AdmContext element.
 *
 * This function creates a new AdmContext element. Its root id node will be
 * initialized using the passed name and serial. The returned AdmContext will
 * be given a ref for the caller.
 *
 * @brief         AdmContext constructor
 * @param  name   The name of the new AdmContext.
 * @param  serial The serial number of the new AdmContext.
 * @param  error  Optional pointer to AdmError pointer.
 * @return        The pointer to the new AdmContext element or @c NULL on
 *                failure.
 */
AdmContext * AdmContext_new                 (const char  *name,
					     unsigned int serial,
					     AdmError **error);

/**
 * AdmContext_ref: ref an AdmContext element.
 *
 * Increases the refcount of an AdmContext element
 *
 * @brief         AdmContext reference increment
 * @param context The AdmContext to increase the reference to.
 * @param error   Optional pointer to AdmError pointer.
 * @return The AdmContext
 */
AdmContext * AdmContext_ref                 (AdmContext   *context,
					     AdmError **error);

/**
 * AdmContext_unref: unref an AdmContext element.
 *
 * Unrefs an AdmContext. This will free the context, and all its children,
 * when the ref count reaches 0. It is an error for an entity which does not
 * own a ref to a context to call AdmContext_unref. AdmContext refs can be
 * acquired only by calls to AdmContext_new, AdmContext_copy,
 * AdmContext_merge, AdmContext_fromXML, and AdmContext_ref.
 *
 * @brief         AdmContext reference decrement       
 * @param context The AdmContext to decrease the reference to.
 * @param error   Optional pointer to AdmError pointer.
 */
void         AdmContext_unref               (AdmContext   *context,
					     AdmError **error);


/* context merge, copy, and flatten */
/**
 * This function creates a full depth copy of an AdmContext element.
 * It is one of the AdmContext refing functions.
 *
 * @brief          AdmContext copy operator
 * @param  context The pointer to the AdmContext element to be copied.
 * @param  error   Optional pointer to AdmError pointer.
 * @return         The pointer to the copied AdmContext element.
 */
AdmContext * AdmContext_copy                (AdmContext  *context,
					     AdmError **error);

/**
 * AdmContext_merge: merge two AdmContext elements into a third.
 * 
 * This is the heart function of the Alchemist. It performs the
 * context merge algorithm upon two contexts, and returns the result.
 * It is also one of the refing functions, as the returned child is
 * refed for the caller.
 * 
 * @brief           AdmContext merge operator
 * @param  name     The name of the new AdmContext.
 * @param  serial   The serial number of the new AdmContext.
 * @param  contextA The pointer to the first AdmContext element.
 * @param  contextB The pointer to the second AdmContext element.
 * @param  error    Optional pointer to an AdmError pointer.
 * @return          The pointer to the new AdmContext element which contains
 *                  the result of the merge operation of contextA and contextB
 *                  or @c NULL on failure.
 */
AdmContext * AdmContext_merge               (const char  *name,
					     unsigned int serial,
					     AdmContext   *contextA,
					     AdmContext   *contextB,
					     AdmError **error);

/**
 * AdmContext_flatten: flatten an AdmContext element.
 *
 * This function strips all masks and AdmCopy elements from an
 * AdmContext element.
 *
 * @brief         AdmContext flatten operator
 * @param context The pointer to the AdmContext element.
 * @param error   Optional pointer to AdmError pointer.
 */
void         AdmContext_flatten             (AdmContext   *context,
					     AdmError **error);

/**
 * AdmContext_strip: strip an AdmContext's id tree.
 *
 * This function strips all ids except for the root id from an AdmContext's id tree.
 *
 * @brief         AdmContext strip function
 * @param context The pointer to the AdmContext element.
 * @param error   Optional pointer to AdmError pointer.
 */
void         AdmContext_strip               (AdmContext   *context,
					     AdmError **error);


/* context encode and decode */
char *       AdmContext_toXML               (AdmContext   *context,
					     AdmError **error);
AdmContext * AdmContext_fromXML             (const char  *xml,
					     AdmError **error);
int	     AdmContext_readFromFile	    (AdmContext  **context,
					     const char   *path,
					     AdmError **error);

int	     AdmContext_writeToFile	    (AdmContext   *context,
					     const char   *path,
					     int	   mode,
					     uid_t	   uid,
					     gid_t	   gid,
					     AdmError **error);

/* context mask manuipulators */
/**
 * AdmContext_getMask: Gets a mask element from an AdmContext by index.
 *
 * This function is used to request the index'th mask of an AdmContext
 * element's mask set. The string returned must be considered const and
 * volatile, and if it is needed for long periods of time, the programmer
 * should copy it.
 *
 * Specifically, it is guranteed for only so long as the AdmContext element
 * is refed, and the mask is not cleared.
 *
 * @brief         AdmContext mask set accessor
 * @param context The AdmContext to use.
 * @param index   The index of the mask set element to get.
 * @param error   Optional pointer to AdmError pointer.
 * @return        The mask set element AdmRefStr.
 */
AdmRefStr * AdmContext_getMask           (AdmContext   *context,
					     unsigned int index,
					     AdmError **error);

/**
 * AdmContext_setMask: set a mask in an AdmContext element's mask set.
 * 
 * This function adds a mask path to an AdmContext element's mask set.
 * It will quietly handle duplicate paths. Blank paths will not be added.
 *
 * @brief         AdmContext mask set mutator
 * @param context The AdmContext to use.
 * @param path    The string of the mask set element to be added.
 * @param error   Optional pointer to AdmError pointer.
 * @return -1 on error, 0 on success
 * @note          This could be made much faster, if we searched the set only
 *                once.
 */
int         AdmContext_setMask           (AdmContext  *context,
					     const char  *path,
					     AdmError **error);

/**
 * AdmContext_clearMaskByIndex: remove a mask entry from an AdmContext by
 * index.
 * 
 * This function removes a mask entry from an AdmContext element by searching
 * against the passed index. It is an error to index past the end of the mask
 * set.
 * 
 * @brief         AdmContext mask set element index remove
 * @param context The AdmContext to use.
 * @param index   The index of the mask set element to remove.
 * @param error   Optional pointer to AdmError pointer.
 * @return -1 on error, 0 on success
 */
int         AdmContext_clearMaskByIndex  (AdmContext  *context,
					     unsigned int index,
					     AdmError **error);

/**
 * AdmContext_clearMaskByPath: remove a mask entry from an AdmContext by
 * string.
 * 
 * This function removes a mask entry from an AdmContext element by searching
 * against the passed string. 
 * 
 * @brief         AdmContext mask set element data path remove
 * @param context The AdmContext to use.
 * @param path     The data path of the mask set element to remove.
 * @param error   Optional pointer to AdmError pointer.
 * @return -1 on error, 0 on success
 */
int         AdmContext_clearMaskByPath   (AdmContext  *context,
					     const char  *path,
					     AdmError **error);

/**
 * AdmContext_getNumMasks: Gets the number of masks in an AdmContext's
 * mask set.
 *
 * This function returns the number of masks in an AdmContext's mask set.
 *
 * @brief          AdmContext mask set length aggregator
 * @param  context The AdmContext to use.
 * @param  error   Optional pointer to AdmError pointer.
 * @return         The number of mask set element of the given AdmContext.
 */
unsigned int AdmContext_getNumMasks       (AdmContext  *context,
					     AdmError **error);


/* identity related funcs */
/**
 * AdmContext_getIdentityRoot: Get the root AdmIdentity element of an
 * AdmContext's id tree.
 * 
 * This function gets the root node AdmIdentity element of an AdmContext
 * element's id tree.
 *
 * @brief          AdmContext identity root accessor
 * @param  context The AdmContext of which we want to get the root AdmIdentity.
 * @param  error   Optional pointer to AdmError pointer.
 * @return         The pointer to the root AdmIdentity element.
 */
AdmIdentity *AdmContext_getIdentityRoot     (AdmContext   *context,
					     AdmError **error);
#define AdmContext_getIdentityRoot_b(CONTEXT, ERROR) \
({ \
	AdmError **__err = (ERROR); \
	AdmIdentity *__id; \
	__id = AdmContext_getIdentityRoot((CONTEXT), __err); \
	if (__id != NULL) \
		AdmIdentity_unref(__id, __err); \
	__id; \
})

/**
 * AdmIdentity_getParentA: Get the A parent of an AdmIdentity element.
 * 
 * This function returns the A parent of an AdmIdentity element, if
 * it exists. Otherwise, it returns NULL.
 * 
 * @brief           AdmIdentity parentA accessor
 * @param  identity The AdmIdentity of which to get the first parent.
 * @param  error    Optional pointer to AdmError pointer.
 * @return          The AdmIdentity pointer to the first parent.
 */
AdmIdentity *AdmIdentity_getParentA         (AdmIdentity  *identity,
					     AdmError **error);
#define AdmIdentity_getParentA_b(ID, ERROR) \
({ \
	AdmError **__err = (ERROR); \
 	AdmIdentity *__id; \
	__id = AdmIdentity_getParentA((ID), __err); \
	if (__id != NULL) \
		AdmIdentity_unref(__id, __err); \
	__id; \
})

/**
 * AdmIdentity_getParentB: Get the B parent of an AdmIdentity element.
 * 
 * This function returns the B parent of an AdmIdentity element, if
 * it exists. Otherwise, it returns NULL.
 * 
 * @brief           AdmIdentity parentB accessor
 * @param  identity The AdmIdentity of which to get the second parent.
 * @param  error    Optional pointer to AdmError pointer.
 * @return          The AdmIdentity pointer to the first parent.
 */
AdmIdentity *AdmIdentity_getParentB         (AdmIdentity  *identity,
					     AdmError **error);
#define AdmIdentity_getParentB_b(ID, ERROR) \
({ \
	AdmError **__err = (ERROR); \
 	AdmIdentity *__id; \
	__id = AdmIdentity_getParentB((ID), __err); \
	if (__id != NULL) \
		AdmIdentity_unref(__id, __err); \
	__id; \
})

/**
 * AdmIdentity_getName: Get the name of an AdmIdentity element.
 * 
 * This function is used to request the name of an AdmIdentity element.
 * The string returned must be considered const and volatile, and if
 * it is needed for long periods of time, the programmer should copy it.
 *
 * Specifically, it is guranteed for only so long as the AdmCopy element
 * is refed, and its value is not changed.
 * 
 * @brief           AdmIdentity name accessor
 * @param  identity The AdmIdentity of which to get the name.
 * @param  error    Optional pointer to AdmError pointer.
 * @return          The name of the given AdmIdentity in an AdmRefStr.
 */
AdmRefStr    *AdmIdentity_getName            (AdmIdentity  *identity,
					     AdmError **error);

/**
 * AdmIdentity_setName: Set the name of an AdmIdentity element.
 * 
 * This function sets the name of an AdmIdentity element.
 * 
 * @brief           AdmIdentity name mutator
 * @param  identity The AdmIdentity of which to set the name.
 * @param  name     The new name of the AdmIdentity.
 * @param  error    Optional pointer to AdmError pointer.
 * @return -1 on error, 0 on success
 */
int         AdmIdentity_setName            (AdmIdentity  *identity,
					     const char   *name,
					     AdmError **error);

/**
 * AdmIdentity_getSerial: Get the serial number of an AdmIdentity element.
 * 
 * This function gets the serial number of an AdmIdentity element.
 * 
 * @brief           AdmIdentity serial number accessor
 * @param  identity The AdmIdentity of which to get the serial number.
 * @param  error    Optional pointer to AdmError pointer.
 * @return          The serial number of the given AdmIdentity.
 */
unsigned int AdmIdentity_getSerial         (AdmIdentity  *identity,
					     AdmError **error);

/**
 * AdmIdentity_setSerial: Set the serial number of an AdmIdentity element.
 * 
 * This function sets the serial number of an AdmIdentity element.
 * 
 * @brief           AdmIdentity serial number mutator
 * @param  identity The AdmIdentity of which to set the serial number.
 * @param  name     The new serial number of the AdmIdentity.
 * @param  error    Optional pointer to AdmError pointer.
 * @return -1 on error, 0 on success
 */
int         AdmIdentity_setSerial          (AdmIdentity  *identity,
					     unsigned int  serial,
					     AdmError **error);

/**
 * AdmIdentity_ref: ref an AdmIdentity element.
 *
 * This function refs an AdmIdentity element. It is the only way to
 * acquire a ref to an AdmIdentity element.
 * 
 * @brief          AdmIdentity reference increment
 * @param identity The AdmIdentity to increase the reference to.
 * @param error    Optional pointer to AdmError pointer.
 * @return The AdmIdentity (for convienience).
 */
AdmIdentity *AdmIdentity_ref                (AdmIdentity  *identity,
					     AdmError **error);

/**
 * AdmIdentity_unref: unref an AdmIdentity element.
 * 
 * This function unrefs an AdmIdentity element, and frees it if the
 * refcount reaches 0. It must only be called by entities which own
 * a ref to the AdmIdentity element. AdmIdentity refs can only be
 * aquired by calling AdmIdentity_ref.
 * 
 * @brief          AdmIdentity reference decrement
 * @param identity The AdmIdentity to decrease the reference to.
 * @param error    Optional pointer to AdmError pointer.
 */
void         AdmIdentity_unref              (AdmIdentity  *identity,
					     AdmError **error);

/**
 * AdmIdentity_getContext: Get the context of an AdmIdentity element.
 * 
 * This function returns a pointer to the AdmContext in which this
 * AdmIdentity element exists.
 *
 * @brief        AdmIdentity context accessor
 * @param  id    The pointer to the AdmIdentity element.
 * @param  error Optional pointer to AdmError pointer.
 * @return       The pointer to the AdmContext of the given AdmIdentity.
 */
AdmContext * AdmIdentity_getContext         (AdmIdentity  *identity,
					     AdmError **error);
#define AdmIdentity_getContext_b(ID, ERROR) \
({ \
	AdmError **__err = (ERROR); \
	AdmContext *__ctx; \
	__ctx = AdmIdentity_getContext((ID), __err); \
	if (__ctx != NULL) \
		AdmContext_unref(__ctx, __err); \
	__ctx; \
})

/**
 * AdmIdentity_isAlive: check the lifecycle status of an AdmIdentity element.
 * 
 * This function checks the lifecycle status of an AdmIdentity element. It
 * is used to tell if an element is still linked against an AdmContext,
 * and is 'alive'; or if it is not, and is 'dead'.
 *
 * A dead element must have a reference held against it that is not its
 * container's reference.
 * 
 * @brief          AdmIdentity reference check
 * @param identity The AdmIdentity to check.
 * @param error    Optional pointer to AdmError pointer.
 * @return         @c TRUE if Identity is still referenced, @c FALSE otherwise.
 */
int          AdmIdentity_isAlive            (AdmIdentity  *identity,
					     AdmError **error);


/* data group structure funcs */
/**
 * AdmContext_getDataRoot: Get the root AdmData element of an AdmContext's
 * data tree.
 *
 * This function gets the root AdmData element of an AdmContext's data tree.
 *
 * @brief          AdmContext data root accessor
 * @param  context The AdmContext of which we want to get the AdmData root.
 * @param  error   Optional pointer to AdmError pointer.
 * @return         The pointer to the root AdmData element.
 */
AdmData *    AdmContext_getDataRoot         (AdmContext   *context,
					     AdmError **error);
#define AdmContext_getDataRoot_b(CONTEXT, ERROR) \
({ \
	AdmError **__err = (ERROR); \
	AdmData *__data; \
	__data = AdmContext_getDataRoot((CONTEXT), __err); \
	if (__data != NULL) \
		AdmData_unref(__data, __err); \
	__data; \
})

/**
 * AdmData_getContainer: Get the container of an AdmData element.
 * 
 * This function returns a pointer to the AdmData element which 
 * contains this AdmData element. For the root node, it will
 * return a pointer to the AdmData element it was passed.
 * 
 * @brief        AdmData container accessor
 * @param  data  The AdmData of which we want to get the container (parent).
 * @param  error Optional pointer to AdmError pointer.
 * @return       The container/parent of the given AdmData element.
 */
AdmData *    AdmData_getContainer           (AdmData      *data,
					     AdmError **error);
#define AdmData_getContainer_b(DATA, ERROR) \
({ \
	AdmError **__err = (ERROR); \
	AdmData *__data; \
	__data = AdmData_getContainer((DATA), __err); \
	if (__data != NULL) \
		AdmData_unref(__data, __err); \
	__data; \
})

/**
* @brief	get the previous sibling of an AdmData element
* @param data	The AdmData element
* @param wrap	specify if we should wrap round the begining of the list
* @param  error Optional pointer to AdmError pointer.
* @return 	the previous AdmData element
*/
AdmData *    AdmData_getPrev		    (AdmData      *data,
					     int	   wrap,
					     AdmError **error);
#define AdmData_getPrev_b(DATA, WRAP, ERROR) \
({ \
	AdmError **__err = (ERROR); \
	AdmData *__data; \
	__data = AdmData_getPrev((DATA), (WRAP), __err); \
	if (__data != NULL) \
		AdmData_unref(__data, __err); \
	__data; \
})

/**
* @brief	get the next sibling of an AdmData element
* @param data	The AdmData element
* @param wrap	specify if we should wrap round the end of the list
* @param error	Optional pointer to AdmError pointer.
* @return 	the next AdmData element
*/
AdmData *    AdmData_getNext		    (AdmData      *data,
					     int	   wrap,
					     AdmError **error);
#define AdmData_getNext_b(DATA, WRAP, ERROR) \
({ \
	AdmError **__err = (ERROR); \
	AdmData *__data; \
	__data = AdmData_getNext((DATA), (WRAP), __err); \
	if (__data != NULL) \
		AdmData_unref(__data, __err); \
	__data; \
})

/**
 * AdmData_getContext: Get the context of an AdmData element.
 * 
 * This function returns a pointer to the AdmContext in which this
 * AdmData element exists.
 * 
 * @brief        AdmData context accessor
 * @param  data  The AdmData of which we want to get the context.
 * @param  error Optional pointer to AdmError pointer.
 * @return       The context of the given AdmData element.
 */
AdmContext * AdmData_getContext             (AdmData      *data,
					     AdmError **error);
#define AdmData_getContext_b(DATA, ERROR) \
({ \
	AdmError **__err = (ERROR); \
	AdmContext *__ctx; \
	__ctx = AdmData_getContext((DATA), __err); \
	if (__ctx != NULL) \
		AdmContext_unref(__ctx, __err); \
	__ctx; \
})
	
/**
 * AdmContext_getDataByPath: Gets an AdmData element from an AdmContext by path.
 *
 * This function does path lookups against AdmContext element's data trees. It
 * will give an error if no matching AdmData element is found.
 *
 * @brief          AdmData path lookup
 * @param  context The AdmContext we want to search.
 * @param  path    The data path of the AdmData element to look up.
 * @param  error   Optional pointer to AdmError pointer.
 * @return         The pointer to the found AdmData element or @c NULL on
 *                 failure.
 */
AdmData *    AdmContext_getDataByPath       (AdmContext   *context,
                                             const char  *path,
					     AdmError **error);
#define AdmContext_getDataByPath_b(CONTEXT, PATH, ERROR) \
({ \
	AdmError **__err = (ERROR); \
	AdmData *__data; \
	__data = AdmContext_getDataByPath((CONTEXT), (PATH), __err); \
	if (__data != NULL) \
		AdmData_unref(__data, __err); \
	__data; \
})

/* data group meta funcs */
/**
 * AdmData_getType: Get the type of an AdmData element.
 * 
 * This function gets the type of an AdmData element
 * 
 * @brief        AdmData type accessor
 * @param  data  The AdmData of which we want to get the data type.
 * @param  error Optional pointer to AdmError pointer.
 * @return       The data type of the given AdmData element.
 */
AdmDataType  AdmData_getType                (AdmData      *data,
					     AdmError **error);

/**
 * AdmData_ref: ref an AdmData element.
 * 
 * This function refs an AdmData element. It is the only way to
 * aquire a ref to an AdmData element.
 * 
 * @brief       AdmData reference increment
 * @param data  The AdmData to increase the reference to.
 * @param error Optional pointer to AdmError pointer.
 * @return The AdmData element
 */
AdmData *   AdmData_ref                    (AdmData      *data,
					     AdmError **error);

/**
 * AdmData_unref: unref an AdmData element.
 * 
 * This function unrefs an AdmData element, and frees it if the
 * refcount reaches 0. It must only be called by entities which
 * own a ref to the AdmData element. AdmData refs can only be
 * aquired by calling AdmData_ref.
 * 
 * @brief       AdmData reference decrement
 * @param data  The AdmData to decrease the reference to.
 * @param error Optional pointer to AdmError pointer.
 */
void         AdmData_unref                  (AdmData      *data,
					     AdmError **error);

/**
 * AdmData_isAlive: check the lifecycle status of an AdmData element.
 * 
 * This function checks the lifecycle status of an AdmData element. It
 * is used to tell if an element is still linked against an AdmContext,
 * and is 'alive'; or if it is not, and is 'dead'.
 *
 * A dead element must have a reference held against it that is not its
 * container's reference.
 * 
 * @brief       AdmData reference check
 * @param data  The AdmData to check.
 * @param error Optional pointer to AdmError pointer.
 * @return      @c TRUE if Data is still referenced, @c FALSE otherwise.
 */
int          AdmData_isAlive                (AdmData      *data,
					     AdmError **error);

/**
 * AdmData_unlink: Unlink an AdmData element from its container.
 * 
 * This function recursively unlinks an AdmData element from its
 * containing AdmData element. It unrefs each element which it
 * unlinks, as they loose their parents reference. It is an error
 * to unlink the root node, and will fail.
 * 
 * @brief       AdmData unlink operator
 * @param data  The AdmData to unlink.
 * @param error Optional pointer to AdmError pointer.
 * @return -1 on error, 0 on success
 */
int         AdmData_unlink                 (AdmData      *data,
					     AdmError **error);

/**
* Checks a string to determine if it is a valid AdmData name.
* A valid name is of the form:
* Name = (Letter | '_') (Letter | Digit | '_' | '-' | '.'){0,254}
* Letter = ['a' - 'z'] | ['A' - 'Z'] 
* Digit = ['0' - '9']
* @brief	Checks a string for name validity
* @param name	The string to check.
* @return	@c 1 if the name is valid, @c 0 if the name is not.
**/
int         AdmData_validName              (const char   *name);

/**
 * AdmData_getName: Get the name of an AdmData element.
 * 
 * This function is used to request the name of an AdmData element.
 * The string returned must be considered const and volatile, and if
 * it is needed for long periods of time, the programmer should copy it.
 *
 * Specifically, it is guranteed for only so long as the AdmData element
 * is refed, and its name is not changed.
 * 
 * @brief        AdmData name accessor
 * @param  data  The AdmData of which we want to get the name.
 * @param  error Optional pointer to AdmError pointer.
 * @return       The name of the given AdmData element.
 */
AdmRefStr  * AdmData_getName                (AdmData      *data,
					     AdmError **error);

/**
 * AdmData_setName: Set the name of an AdmData element.
 * 
 * This function sets the name of an AdmData element to a copy
 * of the passed string.
 *
 * It will not alter the root node, and it is an error to attempt it.
 *
 * @brief        AdmData name mutator
 * @param  data  The AdmData of which we want to set the name.
 * @param  name  The new name of the AdmData element.
 * @param  error Optional pointer to AdmError pointer.
 * @return -1 on error, 0 on success
 */
int         AdmData_setName                (AdmData      *data,
					     const char   *name,
					     AdmError **error);

/**
 * AdmData_getSource: Get the source of an AdmData element.
 * 
 * This function is used to request the source of an AdmData element.
 * The string returned must be considered const and volatile, and if
 * it is needed for long periods of time, the programmer should copy it.
 *
 * Specifically, it is guranteed for only so long as the AdmData element
 * is refed, and its source is not changed. The source of the root data
 * node changes every time the name of the root id node changes.
 * 
 * @brief        AdmData source accessor
 * @param  data  The AdmData of which we want to get the source.
 * @param  error Optional pointer to AdmError pointer.
 * @return       The source of the given AdmData element.
 */
AdmRefStr * AdmData_getSource              (AdmData      *data,
					     AdmError **error);

/**
 * AdmData_setSource: Set the source of an AdmData element.
 * 
 * This function sets the source of an AdmData element to a copy
 * of the passed string.
 *
 * It will not alter the root node, and it is an error to attempt it.
 * 
 * @brief        AdmData source mutator
 * @param  data  The AdmData of which we want to set the source.
 * @param  name  The new source of the AdmData element.
 * @param  error Optional pointer to AdmError pointer.
 * @return -1 on error, 0 on success
 */
int         AdmData_setSource              (AdmData      *data,
					     const char   *source,
					     AdmError **error);

/**
 * AdmData_isProtected: Get the state of an AdmData's protected flag.
 * 
 * This function gets the state of an AdmData's protected flag.
 * 
 * @brief        AdmData protected verification
 * @param  data  The AdmData which we want to check for the protected flag.
 * @param  error Optional pointer to AdmError pointer.
 * @return       @c TRUE if the given AdmData element is protected, @c FALSE
 *               otherwise.
 */
int          AdmData_isProtected            (AdmData      *data,
					     AdmError **error);

/**
 * AdmData_setProtected: Set an AdmData's protected flag.
 * 
 * This function sets the protected flag of an AdmData element
 * to the value passed.
 * 
 * It will not set the flag of the root node, and is an error to
 * attempt it.
 * 
 * @brief        AdmData protected mutator
 * @param  data  The AdmData of which we want to set the protected flag.
 * @param  state The new protection state of the AdmData element.
 * @param  error Optional pointer to AdmError pointer.
 * @return -1 on error, 0 on success
 */
int         AdmData_setProtected           (AdmData      *data,
					     int           state,
					     AdmError **error);


/* list meta funcs */
/**
 * AdmList_isAnonymous: Get the state of an AdmList's anonymous flag.
 * 
 * This function gets the state of an AdmList's anonymous flag.
 * 
 * @brief        AdmList anonymous verification
 * @param  list  The AdmList which we want to check for the anonymous flag.
 * @param  error Optional pointer to AdmError pointer.
 * @return       @c TRUE if the given AdmList element is anonymous, @c FALSE
 *               otherwise.
 */
int          AdmList_isAnonymous            (AdmList      *list,
					     AdmError **error);

/**
 * AdmList_setAnonymous: Set an AdmList's anonymous flag.
 * 
 * This function sets the anonymous flag of an AdmList element
 * to the value passed. It will not allow the flag to be cleared
 * if it will cause a name collide, and will raise an error.
 * 
 * It will not set the flag of the root node, and is an error to
 * attempt it.
 * 
 * @brief        AdmList anonymous mutator
 * @param  list  The AdmList of which we want to set the anonymous flag.
 * @param  state The new anonymous state of the AdmList element.
 * @param  error Optional pointer to AdmError pointer.
 * @return -1 on error, 0 on success
 */
int         AdmList_setAnonymous           (AdmList      *list,
					     int           state,
					     AdmError **error);

/**
 * AdmList_isAtomic: Get the state of an AdmList's atomic flag.
 * 
 * This function gets the state of an AdmList's atomic flag.
 * 
 * @brief        AdmList atomic verification
 * @param  list  The AdmList which we want to check for the atomic flag.
 * @param  error Optional pointer to AdmError pointer.
 * @return       @c TRUE if the given AdmList element is atomic, @c FALSE
 *               otherwise.
 */
int          AdmList_isAtomic               (AdmList      *list,
					     AdmError **error);

/**
 * AdmList_setAtomic: Set an AdmList's atomic flag.
 * 
 * This function sets the atomic flag of an AdmList element
 * to the value passed.
 * 
 * It will not set the flag of the root node, and is an error to
 * attempt it.
 * 
 * @brief        AdmList atomic mutator
 * @param  list  The AdmList of which we want to set the atomic flag.
 * @param  state The new atomic state of the AdmList element.
 * @param  error Optional pointer to AdmError pointer.
 * @return -1 on error, 0 on success
 */
int         AdmList_setAtomic              (AdmList      *list,
					     int           state,
					     AdmError **error);

/**
 * AdmList_isPrepend: Get the state of an AdmList's prepend flag.
 * 
 * This function gets the state of an AdmList's prepend flag.
 * 
 * @brief        AdmList prepend verification
 * @param  list  The AdmList which we want to check for the prepend flag.
 * @param  error Optional pointer to AdmError pointer.
 * @return       @c TRUE if the given AdmList element is prepend, @c FALSE
 *               otherwise.
 */
int          AdmList_isPrepend              (AdmList      *list,
					     AdmError **error);

/**
 * AdmList_setPrepend: Set an AdmList's prepend flag.
 * 
 * This function sets the prepend flag of an AdmList element to the
 * value passed.
 *
 * It will not set the flag of the root node, and is an error to
 * attempt it.
 * 
 * @brief        AdmList prepend mutator
 * @param  list  The AdmList of which we want to set the prepend flag.
 * @param  state The new prepend state of the AdmList element.
 * @param  error Optional pointer to AdmError pointer.
 * @return -1 on error, 0 on success
 */
int         AdmList_setPrepend             (AdmList      *list,
					     int           state,
					     AdmError **error);


/* list child funcs */
/**
 * AdmList_getNumData: Get the size of an AdmList's child set.
 * 
 * This function gets the number of children held in an AdmList's
 * child set.
 * 
 * @brief        AdmList length aggregator
 * @param  list  The AdmList which we want to count.
 * @param  error Optional pointer to AdmError pointer.
 * @return       The number of children inside the given list.
 */
unsigned int AdmList_getNumData         (AdmList      *list,
					     AdmError **error);

/**
 * AdmList_getDataByIndex: Get a child of an AdmList element by its index.
 * 
 * This function seeks on an AdmList's child set, and returns its index'th
 * child. It is an error to seek on an index greater >= the list length,
 * since the first element is stored at index 0.
 * 
 * @brief        AdmList child index accessor
 * @param  list  The AdmList which we want to access.
 * @param  index The index of the child we want to get.
 * @param  error Optional pointer to AdmError pointer.
 * @return       The pointer to requested AdmData element of the list.
 */
AdmData *    AdmList_getDataByIndex        (AdmList      *list,
					     unsigned int  index,
					     AdmError **error);
#define AdmList_getDataByIndex_b(LIST, INDEX, ERROR) \
({ \
	AdmError **__err = (ERROR); \
	AdmData *__data; \
	__data = AdmList_getDataByIndex((LIST), (INDEX), __err); \
	if (__data != NULL) \
		AdmData_unref(__data, __err); \
	__data; \
})

/**
 * AdmList_getDataByName: Get a child od an AdmList element by its name.
 * 
 * This function seeks againts an AdmList's child set, and returns the first
 * child whose name matches the search name. It raises a AdmError if no name is
 * found.
 * 
 * @brief        AdmList child name accessor
 * @param  list  The AdmList which we want to access.
 * @param  name  The name of the child we want to get.
 * @param  error Optional pointer to AdmError pointer.
 * @return       The pointer to requested AdmData element of the list.
 */
AdmData *    AdmList_getDataByName         (AdmList      *list,
					     const char   *name,
					     AdmError **error);
#define AdmList_getDataByName_b(LIST, NAME, ERROR) \
({ \
	AdmError **__err = (ERROR); \
	AdmData *__data; \
	__data = AdmList_getDataByName((LIST), (NAME), __err); \
	if (__data != NULL) \
		AdmData_unref(__data, __err); \
	__data; \
})

/**
 * AdmList_getDataByPath: Get a child od an AdmList element by its path.
 * 
 * This function seeks againts an AdmList's child set, and returns the first
 * child whose path matches the search path. It raises a AdmError if no path is
 * found.
 * 
 * @brief        AdmList child data path accessor
 * @param  list  The AdmList which we want to access.
 * @param  path  The data path of the child we want to get.
 * @param  error Optional pointer to AdmError pointer.
 * @return       The pointer to requested AdmData element of the list.
 */
AdmData *    AdmList_getDataByPath         (AdmList      *list,
					     const char   *path,
					     AdmError **error);
#define AdmList_getDataByPath_b(LIST, PATH, ERROR) \
({ \
	AdmError **__err = (ERROR); \
	AdmData *__data; \
	__data = AdmList_getDataByPath((LIST), (PATH), __err); \
	if (__data != NULL) \
		AdmData_unref(__data, __err); \
	__data; \
})

/**
 * AdmList_addData: add a new child to an AdmList element.
 * 
 * This function appends a new child of the given type and name to
 * the end of the AdmList. It returns a pointer to the new AdmData
 * element. Name collides in non-anonymous lists are errors. 
 *
 * Note that new AdmCopy elements will have the invalid reference ""
 * as their initial value, and must be changed.
 *
 * The new child is refed, but the ref is owned by its AdmList
 * element, and no ref is implied to the accessor.
 *  
 * @brief        AdmList add child operator
 * @param  list  The AdmList to which we want to add a new child.
 * @param  type  The AdmData type of the new child to be added.
 * @param  name  The name of the new child to be added.
 * @param  error Optional pointer to AdmError pointer.
 * @return       The pointer to the newly added AdmData element.
 */
AdmData *    AdmList_addData               (AdmList      *list,
					     AdmDataType   type,
					     const char   *name,
					     AdmError **error);
#define AdmList_addData_b(LIST, TYPE, NAME, ERROR) \
({ \
	AdmError **__err = (ERROR); \
	AdmData *__data; \
	__data = AdmList_addData((LIST), (TYPE), (NAME), __err); \
	if (__data != NULL) \
		AdmData_unref(__data, __err); \
	__data; \
})

/**
 * AdmList_moveData: reposition an AdmData element in its containing AdmList.
 * 
 * This function extracts an AdmData element from its containing list, and
 * then reinserts it at the index value passed. It is an error to give it an
 * illegal index value.
 * 
 * @brief        AdmList child move operator
 * @param  list  The AdmList which we want to move in.
 * @param  data  The AdmData pointer of the child we want to move.
 * @param  index The index to where we want to move the child.
 * @param  error Optional pointer to AdmError pointer.
 * @return -1 on error, 0 on success
 */
int         AdmList_moveData              (AdmList      *list,
					     AdmData      *data,
					     unsigned int  index,
					     AdmError **error);

/**
 * AdmList_copyData: copy a AdmData element into an AdmList element.
 * 
 * This function recursively copies an AdmData tree from the passed data node,
 * and inserts the result onto the end of the passed AdmList. It checks for
 * name collides in Non-anonymous lists, and will not copy a root node; both
 * are errors.
 *
 * @brief        AdmList copy data operator
 * @param  list  The AdmList to append the data to.
 * @param  data  The AdmData element to recursively copy and append to the list.
 * @param  error Optional pointer to AdmError pointer.
 * @return       The pointer to the newly copied and appended AdmData element.
 */
AdmData *    AdmList_copyData               (AdmList      *list,
					     AdmData      *data,
					     AdmError **error);
#define AdmList_copyData_b(LIST, DATA, ERROR) \
({ \
	AdmError **__err = (ERROR); \
	AdmData *__data; \
	__data = AdmList_copyData((LIST), (DATA), __err); \
	if (__data != NULL) \
		AdmData_unref(__data, __err); \
	__data; \
})

/**
 * AdmList_renameCopyData: copy and rename an AdmData element into an AdmList element.
 * 
 * This function recursively copies an AdmData tree from the passed data node,
 * and inserts the result onto the end of the passed AdmList, using the
 * specified name. It checks for name collides in Non-anonymous lists, and will
 * not copy a root node; both are errors.
 *
 * @brief        AdmList copy data operator
 * @param  list  The AdmList to append the data to.
 * @param  data  The AdmData element to recursively copy and append to the list.
 * @param  name  The new name to use.
 * @param  error Optional pointer to AdmError pointer.
 * @return       The pointer to the newly copied and appended AdmData element.
 */
AdmData *    AdmList_renameCopyData      (AdmList      *list,
					     AdmData      *data,
					     const char *name,
					     AdmError **error);
#define AdmList_renameCopyData_b(LIST, DATA, NAME, ERROR) \
({ \
	AdmError **__err = (ERROR); \
	AdmData *__data; \
	__data = AdmList_renameCopyData((LIST), (DATA), (NAME), __err); \
	if (__data != NULL) \
		AdmData_unref(__data, __err); \
	__data; \
})

/**
 * AdmData_getPos: Get the position of an AdmData node in its containing list.
 * 
 * This function returns the position that an AdmData node holds in its
 * containing list. The position of the root node is defined to be 0.
 * 
 * @brief        AdmList child position operator
 * @param  data  The AdmData of which we want to know the index.
 * @param  error Optional pointer to AdmError pointer.
 * @return       The index of the data element in it's parent list.
 */
unsigned int AdmData_getPos                 (AdmData      *data,
					     AdmError **error);


/* scalar functions */
/**
 * Gets the integer value stored in an AdmInt node.
 * @brief get an AdmInt's value
 * @param int_scalar a pointer to an AdmInt node
 * @param error an optional pointer to an AdmError pointer
 * @return the value of the node
 * @see AdmInt
 * @see AdmError
 */
int          AdmInt_getValue                (AdmInt       *int_scalar,
					     AdmError **error);

/**
 * Sets the integer value stored in an AdmInt node.
 * @brief set an AdmInt's value
 * @param int_scalar a pointer to an AdmInt node
 * @param value the integer value to set the node to
 * @param error an optional pointer to an AdmError pointer
 * @return -1 on error, 0 on success
 * @see AdmInt
 * @see AdmError
 */
int         AdmInt_setValue                (AdmInt       *int_scalar,
					     int          value,
					     AdmError **error);

/**
 * Gets the double value stored in an AdmFloat node.
 * @brief get an AdmFloat's value
 * @param float_scalar a pointer to an AdmFloat node
 * @param error an optional pointer to an AdmError pointer
 * @return the double value of the node
 * @see AdmFloat
 * @see AdmError
 */
double       AdmFloat_getValue              (AdmFloat     *float_scalar,
					     AdmError **error);

/**
 * Sets the double value stored in an AdmFloat node.
 * @brief set an AdmFloat's value
 * @param float_scalar a pointer to an AdmFloat element
 * @param value the double value to set the node with
 * @param error an optional pointer to an AdmError pointer
 * @return -1 on error, 0 on success
 * @see AdmFloat
 * @see AdmError
 */
int         AdmFloat_setValue              (AdmFloat     *float_scalar,
					     double        value,
					     AdmError **error);

/**
 * Gets the boolean value stored in an AdmBool node.
 * @brief get an AdmBool's value
 * @param bool_scalar a pointer to an AdmBool node
 * @param error an optional pointer to an AdmError pointer
 * @return the value of the node as an int
 * @see AdmBool
 * @see AdmError
 */
int          AdmBool_getValue               (AdmBool      *bool_scalar,
					     AdmError **error);

/**
 * Sets the boolean value stored in an AdmBool node.
 * @brief set an AdmBool's value
 * @param bool_scalar a pointer to an AdmBool node
 * @param value the value to set the node to as an int
 * @param error an optional pointer to an AdmError pointer
 * @return -1 on error, 0 on success
 * @see AdmBool
 * @see AdmError
 */
int         AdmBool_setValue               (AdmBool      *bool_scalar,
					     int           value,
					     AdmError **error);

/**
 * Get the value of an AdmString element.
 *
 * This function is used to request the string stored in an AdmString
 * element. The string returned must be considered const and volatile,
 * and if it is needed for long periods of time, the programmer should
 * copy it.
 *
 * Specifically, it is guaranteed for only so long as the AdmString element
 * remains refed, and its value is not changed.
 * 
 * @brief get an AdmString's value
 * @param string_scalar The pointer to the AdmString element
 * @param error optional pointer to an AdmError pointer
 * @return An AdmRefStr with the AdmString element's value string.
 **/
AdmRefStr * AdmString_getValue             (AdmString    *string_scalar,
					     AdmError **error);

/**
 * Set the value of an AdmString element.
 * 
 * This function is used to set the value of an AdmString element.
 * It stores a copy of the value string it is passed.
 * 
 * @param string_scalar The pointer to the AdmString element.
 * @param value The pointer to the string.
 * @param error optional pointer to an AdmError pointer
 * @return -1 on error, 0 on success
 */
int         AdmString_setValue             (AdmString    *string_scalar,
					     const char   *value,
					     AdmError **error);

/**
 * AdmBase64_getValue: Get the value of an AdmBase64 element
 * @base64_scalar: a pointer to an AdmBase64 element
 * @error: an optional AdmError
 * 
 * This function is used to request the string stored in an AdmBase64 string
 * element. The string returned must be considered const and volatile, and if
 * it is needed for long periods of time, the programmer should copy it.
 *
 * Return value: a pointer to the AdmBase64 element's value string.
 **/
AdmRefStr * AdmBase64_getValue             (AdmBase64    *base64_scalar,
					     AdmError **error);

/**
 * This function is used to set the value of an AdmBase64 string element.
 * It stores a copy of the value string it is passed.
 *
 * @brief Set the value of an AdmBase64 element
 * @param base64_scalar a pointer to an AdmBase64 element
 * @param value a pointer to a base64 encoded binary string
 * @param error an optional AdmError
 * @return -1 on error, 0 on success
 **/
int         AdmBase64_setValue             (AdmBase64    *base64_scalar,
					     const char   *value,
						AdmError **error);
/**
 * @brief Get the binary value of an AdmBase64 element
 * @param base64_scalar a pointer to an AdmBase64 element
 * @param value a pointer to a void pointer to return the address of the new buffer.
 * @param error an optional AdmError
 * @return the size of the new buffer.
 *
 * This function will decode and return a copy of the binary string stored in
 * a AdmBase64 element. You must free the returned buffer.
 * 
 * Return value: a pointer to the AdmBase64 element's value string.
 **/
size_t	     AdmBase64_getBinValue          (AdmBase64    *base64_scalar,
					     void	 **value,
					     AdmError **error);
/**
 * This function is used to set the binary value of an AdmBase64 string element.
 * It stores an encoded copy of the value string it is passed.
 *
 * @brief Set the value of an AdmBase64 element
 * @param base64_scalar a pointer to an AdmBase64 element
 * @param value a pointer to a binary string
 * @param size the size of the binary string
 * @param error an optional AdmError
 * @return -1 on error, 0 on success
 **/
int	     AdmBase64_setBinValue          (AdmBase64    *base64_scalar,
					     const void	 *value,
					     size_t	  size,
					     AdmError **error);

/**
 * This function is used to request the path of an AdmCopy element.
 * The string returned must be considered const and volatile, and if
 * it is needed for long periods of time, the programmer should copy it.
 *
 * Specifically, it is guranteed for only so long as the AdmCopy element
 * is refed, and its value is not changed.
 * 
 * @brief Get the value of an AdmCopy element
 * @param copy_scalar a pointer to an AdmCopy element
 * @param error an optional GError
 * @return An AdmRefStr containing the AdmCopy's path
 **/
AdmRefStr * AdmCopy_getValue               (AdmCopy      *copy_scalar,
					     AdmError **error);

/**
 * This function is used to set the path of an AdmCopy element.
 * It stores a copy of the string passed to it, after checking
 * its path validity.
 * @brief Set the value of an AdmCopy element
 * @param copy_scalar a pointer to an AdmCopy element
 * @param value a pointer to a valid path string
 * @param error an optional GError
 * @return -1 on error, 0 on success
 **/
int         AdmCopy_setValue               (AdmCopy      *copy_scalar,
					     const char   *value,
					     AdmError **error);

/* blackbox prototypes */
typedef struct AdmBlackBox AdmBlackBox;


/**
* Gets the cache directory the alchemist is currently using.
* @param error an optional pointer to an AdmError pointer
* @return the cache dir
* @note you do not own the return stritg, and must not free it
*/
AdmRefStr * AdmBlackBox_getCacheDir       (AdmError **error);

/**
* Sets the cache directory the alchemist is currently using.
* @param path the cache directory you want to use
* @param error an optional pointer to an AdmError pointer
* @return -1 on failure, 0 on succes
* @note passing NULL as the path resets the path to the default
*/
int          AdmBlackBox_setCacheDir       (const char *path,
					     AdmError **error);

AdmBlackBox *AdmBlackBox_getBox		    (AdmList      *box_cfg,
					     AdmError **error);

typedef struct {
	unsigned char array[1];
} adm_bb_cap_t;
/**
 * Returns a capability vector listing the capabilities of an AdmBlackBox.
 *
 * @brief        AdmBlackBox capability querry
 * @param  box   The AdmBlackBox to check
 * @param  error Optional pointer to AdmError pointer.
 * @return       an adm_bb_cap_t structure
 */
adm_bb_cap_t AdmBlackBox_caps			(AdmBlackBox  *box,
						 AdmError **error);
#define ADM_BBCAP_SUP_WRITE(CAP)	((CAP).array[0] & (1 << 0))
#define ADM_BBCAP_SUP_READ(CAP)		((CAP).array[0] & (1 << 1))
#define ADM_BBCAP_SUP_LISTSERIAL(CAP)	((CAP).array[0] & (1 << 2))
#define ADM_BBCAP_SUP_READSERIAL(CAP)	((CAP).array[0] & (1 << 3))
#define ADM_BBCAP_CACHED(CAP)		((CAP).array[0] & (1 << 4))

/**
 * Increase reference to given AdmBlackBox element.
 *
 * @brief       AdmBlackbox reference increment
 * @param box   The AdmBlackBox to increase the reference to.
 * @param error Optional pointer to AdmError pointer.
 */     
void	     AdmBlackBox_ref		    (AdmBlackBox  *box,
					     AdmError **error);

/**
 * Decrease reference to given AdmBlackBox element.
 *
 * @brief       AdmBlackbox reference decrement
 * @param box   The AdmBlackBox to decrease the reference to.
 * @param error Optional pointer to AdmError pointer.
 */
void	     AdmBlackBox_unref		    (AdmBlackBox  *box,
					     AdmError **error);

/**
 * Tries to read the most current context from the given AdmBlackBox and return
 * it. The BlackBox may not be able to be readable in which case NULL will be
 * returned. Also it may allow caching and do so transparently.
 *
 * @brief        AdmBlackbox read operator
 * @param  box   The AdmBlackBox to read from.
 * @param  error Optional pointer to AdmError pointer.
 * @return       The pointer to the AdmContext retrived through the given box.
 */
AdmContext * AdmBlackBox_read		    (AdmBlackBox  *box,
					     AdmError **error);

/**
 * Creates a list of serial numbers of the given AdmBlackBox subject to the
 * constraints.
 *
 * @brief             AdmBlackbox serial list request
 * @param  box        The AdmBlackBox to get the list of serial numbers from.
 * @param  array      The pointer to the preallocated array of @c int values to
 *                    be filled with the serial numbers.
 * @param  array_len  The size of the perallocated array.
 * @param  top_serial The highest serial number that we request. Only serial
 *                    numbers below that will be returned. Use @c 0 to request
 *                    the newest (highest) serial numbers.
 * @param  error      Optional pointer to AdmError pointer.
 * @return            The amount of serial numbers filled in. Always less or
 *                    equal to the array length.
 * @note              The list of serial numbers can partially come from the
 *                    local cached BlackBoxes as well.
 */
int	     AdmBlackBox_listSerial	    (AdmBlackBox   *box,
					     unsigned int  *array,
					     unsigned int   array_len,
					     unsigned int   top_serial,
					     AdmError **error);

/**
 * Reads a context from the given AdmBlackBox with the specified serial number.
 *
 * @brief         AdmBlackbox serial list request
 * @param  box    The AdmBlackBox to read the context from.
 * @param  serial The serial number of the context we request.
 * @param  error  Optional pointer to AdmError pointer.
 * @return        The pointer to the AdmContext retrived through the given box
 *                or @c NULL if no context with that serial number exists.
 */
AdmContext * AdmBlackBox_readSerial	    (AdmBlackBox  *box,
					     unsigned int  serial,
					     AdmError **error);

/**
 * Write the given AdmContext to the given AdmBlackBox. Does automatic caching
 * if the given AdmBlackBox supports it.
 *
 * @brief          AdmBlackbox write operator
 * @param  box     The AdmBlackBox to write to.
 * @param  context The AdmContext to be written to the box.
 * @param  error   Optional pointer to AdmError pointer.
 * @return         @c 0 on success, all other values indicate an error.
 */
int	     AdmBlackBox_write		    (AdmBlackBox  *box,
					     AdmContext   *context,
					     AdmError **error);

/**
 * Get the serial number of the given AdmBlackBox for the given mark. The
 * whole business with marks allows us to 'tag' cached boxes and to
 * identify if they are dirty or not.
 *
 * @brief            AdmBlackBox mark accessor
 * @param  box       The AdmBlackBox from which to get the marked serial.
 * @param  mark The mark string.
 * @param  error     Optional pointer to AdmError pointer.
 * @return           The serial number of the marked cached box or 0 if no
 *                   marked cached box exists.
 */
unsigned int AdmBlackBox_getMark	    (AdmBlackBox  *box,
					     const char   *mark,
					     AdmError **error);
/**
 * Mark the given AdmBlackBox. The serial number is extracted from the
 * given AdmBlackBox automatically. Any existing mark will be overwritten.
* If an AdmBlackBox is cached, then it is possible to mark it. This provides a mechanism
* for marking the top serial number in the cache down at a given mark, which is a string;
* allowing various applications to check to see if an interface has changed since the last time
* the accessed by doing something along the lines of:
* @code
* AdmBlackBox_setMark(box_p, "app1_mark_string", error);
* ...
* int i;
* if ((AdmBlackBok_listSerial(box_p, &i, 1, 0, NULL) &&
* 	(i != AdmBlackBox_getMark(box_p, "app1_mark_string", NULL)) {
* 	...
* }
* @endcode
* 
 *
 * @brief            AdmBlackBox mark mutator
 * @param  box       The AdmBlackBox which we want to mark.
 * @param  mark The mark string.
 * @param  error     Optional pointer to AdmError pointer.
 * @return           @c 0 on success, all other values indicate an error.
 */
int	     AdmBlackBox_setMark	    (AdmBlackBox  *box,
					     const char   *mark,
					     AdmError **error);

/**
 * Clear the mark for the given AdmBlackBox.
 *
 * @brief            AdmBlackbox mark destructor
 * @param  box       The AdmBlackBox where we want to clear its mark.
 * @param  mark The mark string.
 * @param  error     Optional pointer to AdmError pointer.
 * @return           @c 0 on success, all other values indicate an error.
 */
int	     AdmBlackBox_clearMark	    (AdmBlackBox  *box,
					     const char   *mark,
					     AdmError **error);

/* switchboard */
/**
* Gets the swb directory the alchemist is currently using.
* @param error an optional pointer to an AdmError pointer
* @return the swb dir
* @note you do not own the return stritg, and must not free it
*/
AdmRefStr * AdmSwitchboard_getSwbDir       (AdmError **error);

/**
* Sets the swb directory the alchemist is currently using.
* @param path the swb directory you want to use
* @param error an optional pointer to an AdmError pointer
* @return -1 on failure, 0 on succes
* @note passing NULL as the path resets the path to the default
*/
int          AdmSwitchboard_setSwbDir       (const char *path,
					     AdmError **error);

int	     AdmSwitchboard_markNsp (const char   *namespace,
						const char   *subspace,
						const char   *mark,
						AdmError **error);

int	     AdmSwitchboard_markNspWithConfig (
						AdmList      *namespace_cfg,
						const char   *subspace,
						const char   *mark,
						AdmError **error);

int	     AdmSwitchboard_checkNspMark     (const char   *namespace,
						const char   *subspace,
						const char   *mark,
						AdmError **error);

int	     AdmSwitchboard_checkNspMarkWithConfig (
						AdmList      *namespace_cfg,
						const char   *subspace,
						const char   *mark,
						AdmError **error);

/**
* read the given subspace from the given namespace
* @param namespace the namespace to read
* @param subspace the box set to extract, NULL uses the default "config" set
* @param depth the number of boxes in the namespace to read, 0 means all. 
* @param error an optional pointer to an AdmError pointer
*/
AdmContext  *AdmSwitchboard_readNsp   (const char   *namespace,
				     const char   *subspace,
				     unsigned int depth,
					     AdmError **error);

AdmContext  *AdmSwitchboard_readNspWithConfig	(
						AdmList      *namespace_cfg,
						const char   *subspace,
						unsigned int depth,
						AdmError **error);

/**
* writes the given subspace in the given namespace
* @param namespace the namespace to write
* @param subspace the box set to extract, NULL uses the default "config" set
* @param context the context to write
* @param error an optional pointer to an AdmError pointer
*/
int	     AdmSwitchboard_writeNsp  (const char   *namespace,
					     const char   *subspace,
					     AdmContext   *context,
					     AdmError **error);

int	     AdmSwitchboard_writeNspWithConfig	(
						AdmList      *namespace_cfg,
						const char   *subspace,
						AdmContext   *context,
						AdmError **error);

/**
* @brief read a switchboard namespace configuration from the system
* @param namespace the name of the namespace to read
* @param error an optional pointer to an AdmError pointer
* @return an AdmContext that is the namespace configuration
*/
AdmContext  *AdmSwitchboard_readNspCtx  (const char   *namespace,
					       AdmError **error);

/**
* @brief writes a switchboard namespace configuration to the system
* @param namespace the name of the namespace to read
* @param context the configuration context to write
* @param mode the file mode to write the context out with
* @param uid the uid to write the file with, -1 for dont care
* @param gid the gid to write the file with, -1 for dont care
* @param error an optional pointer to an AdmError pointer
* @return -1 on failure, 0 on succes
*/
int	     AdmSwitchboard_writeNspCtx (const char   *namespace,
					       AdmContext   *namespace_ctx,
					       int	     mode,
					       uid_t	     uid,
					       gid_t	     gid,
					       AdmError    **error);

/**
* extract a namespace config from a namespace context
* @param namespace the namespace to extract
* @param config the namespace config context
*/
AdmList     *AdmSwitchboard_getNspCfg   (
					       AdmContext   *namespace_ctx,
						const char   *namespace,
					       AdmError **error);
#define AdmSwitchboard_getNspCfg_b(CTX, NAMESPACE, ERROR) \
({ \
	AdmError **__err = (ERROR); \
	AdmList *__list; \
	__list = AdmSwitchboard_getNspCfg((CTX), (NAMESPACE), __err); \
	if (__list != NULL) \
		AdmData_unref((AdmData *) __list, __err); \
	__list; \
})

/**
* extract a box set from a namespace config
* @param subspace the box set to extract, NULL uses the default "config" set
* @param namespace the namespace used
* @param config the namespace config context
*/
AdmList     *AdmSwitchboard_getSubspCfg      (
					       AdmList   *namespace_cfg,
						const char   *subspace,
					       AdmError **error);
#define AdmSwitchboard_getSubspCfg_b(CTX, SUBSPACE, ERROR) \
({ \
	AdmError **__err = (ERROR); \
	AdmList *__list; \
	__list = AdmSwitchboard_getSubspCfg((CTX), (SUBSPACE), __err); \
	if (__list != NULL) \
		AdmData_unref((AdmData *) __list, __err); \
	__list; \
})


/**
* extract a box_config set from a namespace config
* @param box_name the box to extract
* @param subspace_cfg the subspace config no use
*/
AdmBlackBox *AdmSwitchboard_getBox         (
					       AdmList   *subspace_cfg,
						const char   *box_name,
					       AdmError **error);

#endif /* __ALCHEMIST__ */
